#!/usr/bin/python
from __future__ import division
import random
import sys
import getopt


def writetable(writefile, table, sep = '\t'):
    joinrows = [sep.join(map(str,rowlist))+'\n' for rowlist in table]
    with open(writefile, 'wb') as f:
        f.writelines(joinrows)
    return 

def readtable(readfile, sep=' ', ignoreheader = False):
    with open(readfile) as f:
        if (ignoreheader):
            header = f.readline()
        rows = f.readlines()
    table = [row.rstrip().replace('\"', '').split(sep) for row in rows]
    return table

def read_variants(filename, only_int_pos = False, onlySNPs = False, return_sites_only = False, only_biallelic = False, all_variant_sites = False, maskfile = "", add_header = True):
    print(filename)
    variants_table = readtable(filename, sep = '\t', ignoreheader = True)
    chrom = 'Chr' + variants_table[0][0]
    if all_variant_sites:
        var_sites = []
        for row in variants_table:
            maxlen = max([len(r) for r in row[5:]])
            var_sites.extend(range(int(float(row[2])), int(float(row[2]))+maxlen))
        return var_sites
    if only_int_pos: 
        variants_table = [row for row in variants_table if float(row[2]) % 1 == 0]
    if onlySNPs:
        variants_table = [row for row in variants_table if len(''.join(row[5:]))== len(row)-5 and '0' not in ''.join(row[5:])]
        print(len(variants_table))
    if only_biallelic:
        variants_table = [row for row in variants_table if len(list(set(row[5:]))) == 2 and list(set(row[5:]))[0] in 'ACGT' and list(set(row[5:]))[1] in 'ACGT']
    if len(maskfile) > 0:
        mask = readtable(maskfile, sep = '\t')
        new_vars = []
        var_ind = 0
        mask = [row for row in mask if row[0] == chrom]
        print(len(variants_table))
        for mrow in mask:
            start, end = int(mrow[3]), int(mrow[4])
            while(var_ind<len(variants_table)-1 and int(variants_table[var_ind][2]) < start):
                new_vars.append(variants_table[var_ind])
                var_ind+=1
            while(var_ind<len(variants_table)-1 and int(variants_table[var_ind][2]) <= end):
                var_ind+=1
        variants_table = new_vars[:]
        print(len(variants_table))
    if add_header:
        with open(filename) as f:
            header = f.readline()
        variants_table.insert(0, header.rstrip().split('\t'))
    if return_sites_only:
        variants_table = [int(row[2]) for row in variants_table]
    return variants_table

#founders is a list of founders and chr_lengths is a table with columns chromosome names and chromosome lengths
class SimulateMAGIC:
    def __init__(self, name, founders, minrecomb, maxrecomb, chr_lengths):
        self.founders = founders
        self.chr_lengths = {row[0]:int(row[1]) for row in chr_lengths}
        self.chromosomes = self.chr_lengths.keys()
        self.chromosomes.sort()
        self.name = name
        self.nsegs = random.randrange(minrecomb, maxrecomb)
        nfounders = len(self.founders)
        chrom_counts = [0,0,0,0,0]
        while(0 in chrom_counts or 1 in chrom_counts):
            chroms = [random.choice(self.chromosomes) for i in range(0, self.nsegs)]
            chrom_counts = [chroms.count(chrom) for chrom in self.chromosomes]
        chroms.sort()
        self.chroms = chroms[:]
        curr_acc = random.sample(self.founders, nfounders)
        acc = [random.choice(curr_acc)]
        for i in range(1, self.nsegs):
            acc.append(random.choice(curr_acc[:curr_acc.index(acc[i-1])] + curr_acc[curr_acc.index(acc[i-1])+1:]))
        self.acc = acc[:]
        coords = []
        for j in self.chromosomes:
            fro, to = [], []
            if (chroms.count(j)>1):
                to = random.sample(range(1, self.chr_lengths[j]), chroms.count(j)-1)
                to.sort()
                fro = [t+1 for t in to]
            fro.insert(0, 0)
            to.append(self.chr_lengths[j])
            coords.extend(zip(fro, to))
        self.coords = coords[:]
        self.mosaic = [[self.name, int(self.chroms[i]), self.acc[i], int(str(self.coords[i][0])), int(str(self.coords[i][1]))] for i in range(0,len(self.chroms))]
        self.mosaic = sorted(self.mosaic, key=lambda row: row[1])

def get_sites(mosaic, coverage, variants_dict, writefile, founders, seqerr_percent = 0):
    vars = []
    het_to_homo = {'A':'A', 'C':'C', 'G':'G', 'T':'T', 'R':'A', 'Y':'C', 'K':'G', 'M':'A', 'S':'C', 'W':'T', 'B':'G', 'D':'T', 'H':'A', 'V':'C', 'N':'A'}
    print('Get sites')
    for row in mosaic:
        chrom, acc, start, end = row[1:5]
        print(chrom, acc, start, end)
        chrom = str(chrom)
        curr_vars = [[int(chrom), int(var[2]), var[7], het_to_homo[var[5+founders.index(acc)]]] for var in variants_dict[chrom] if float(var[2])>=int(start) and float(var[2])<=int(end) and len(''.join(var[5:])) == len(var)-5 and '0' not in var[5:]]
        vars.extend(curr_vars)
    vars.sort()
    if (seqerr_percent>0):
        random.shuffle(vars)
        vars_err = vars[0:int(len(vars)*(seqerr_percent/100))]
        vars = vars[int(len(vars)*(seqerr_percent/100)):]
        vars_err = [[chr, pos, ref, random.choice(['A','C','G','T'])]for chr, pos, ref, obs in vars_err]
        vars.extend(vars_err)
        vars.sort()
    if coverage<100:
        n_sites = int((coverage/100) * len(vars))
        frac_vars = random.sample(vars, n_sites)
        frac_vars.sort()
        frac_vars = [['Chr'+str(chrom), pos, ref, var] for chrom, pos, ref, var in frac_vars]
        writetable(writefile, frac_vars, sep = " ")
    else:
        vars = [['Chr'+str(chrom), pos, ref, var] for chrom, pos, ref, var in vars]
        writetable(writefile, vars, sep = " ")

##this function computes new mosaic and gets sites
def new_mosaic_and_sites(n_sim, minrecomb, maxrecomb, variants_dict, sim_dir, seqerr_percent, mosaic_write_file, founders, chr_lengths, coverage):
    mosaics = []
    for i in range(1, n_sim+1):
        name = "SIM."+str(i)
        print(name)
        sim = SimulateMAGIC(name, founders, minrecomb, maxrecomb, chr_lengths)
        mosaic = sim.mosaic
        mosaics.extend(mosaic)
        get_sites(mosaic, coverage, variants_dict, sim_dir+name, founders, seqerr_percent)
    writetable(mosaic_write_file, mosaics, sep = " ")
    
if __name__=='__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:], "s:l:h:v:o:e:m:c:k:f:", ["simulations=", "low_recomb=", "high_recomb=", "variants_dir=", "output_dir=", "error=", "mosaic_file=", "chr_lengths=", "maskfile=", 'coverage_perc='])
    except getopt.GetoptError as err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        sys.exit(2)
    n_sim = 100
    minrecomb = 25
    maxrecomb = 35
    var_dir = './'
    sim_dir = "./"
    seqerr_percent = 0
    mosaic_write_file = "./sim.mosaic.txt"
    chr_lengths = [['1', '30427671'], ['2', '19698289'], ['3', '23459830'], ['4', '18585056'], ['5', '26975502']]
    maskfile = ""
    coverage_percent = 100
    for o, a in opts:
        if o in ['-s', '--simulations']:
            n_sim = int(a)
        elif o in ['-l', '--low_recomb']:
            minrecomb = int(a)
        elif o in ['-h', '--high_recomb']:
            maxrecomb = int(a)
        elif o in ['-v', '--variants_dir']:
            var_dir = a
        elif o in ['-o', '--output_dir']:
            sim_dir = a
        elif o in ['-e', '--error']:
            seqerr_percent = float(a)
        elif o in ['-m', '--mosaic_file']:
            mosaic_write_file = a
        elif o in ['-c', '--chr_lengths']:
            chr_lengths_file = a
            chr_lengths = readtable(chr_lengths_file, ignoreheader = False, sep = '\t')
        elif o in ['-k', '--maskfile']:
            maskfile = a
        elif o in ['-f', '--coverage_perc']:
            coverage_percent = int(a)
            
    variants_dict = {}

    #read in variants_dict
    for i, size in chr_lengths:
        variants_dict[i] = read_variants(var_dir+'chr'+i+'.alleles.txt', only_int_pos = True, onlySNPs = True, return_sites_only = False, only_biallelic = True, all_variant_sites = False, maskfile = maskfile, add_header = True)
    #parse founders

    founders = variants_dict['1'][0][5:]
    print(founders)
    #remove header column
    for i, size in chr_lengths:
        variants_dict[i] = variants_dict[i][1:]

    #get new mosaic and sites
    new_mosaic_and_sites(n_sim, minrecomb, maxrecomb, variants_dict, sim_dir, seqerr_percent, mosaic_write_file, founders, chr_lengths, coverage_percent)

    
    
        
        
